/* ////////////////////////////////////////////////////////////////////////////
** File:      IMUDisp.pde              */
 String VerCon = new String ("IMUDisp   0.9.0 - 01-2013"); //---
/* Author:    Guido Ottaviani-->guido@guiott.com<--
** Description: UDB4 IMU control board remote display for testing
** the original idea for the gauge graphic is on this site: 
** http://angela.homelinux.net:8001/code.html

-------------------------------------------------------------------------------
** Compatible with UDB4 board
-------------------------------------------------------------------------------
Copyright 2013 Guido Ottaviani
guido@guiott.com

    IMUDisp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IMUDisp is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ImuConsole.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------    
/////////////////////////////////////////////////////////////////////////////*/
import processing.serial.*;
PFont letters;
PFont mono;

// UDB4 IMU parameters
int Lat_gps;
int Lon_gps;
int Alt_gps;
int Sog_gps;
int Cog_gps;
int Week_no;
int Tow;
int Hdop;
int Svs;
int[] Rmat = new int[9]; 
int Cpu_load;

String[] SerialList = new String[0];  // declare an empty string for serial ports list 
Serial RS232Port;
int RS232ComPort = 99;
int RS232Bps;
// it will be set false after RS232 configuration to start real communication
boolean PreInitRS232Flag = true;  
boolean StopFlag = true;
boolean RS232FirstInit = true;
String RxErrorText = new String ("--RX OK--");
char RxErrorExpected;
char RxErrorFound;
int RxErrorDispTime = 0;
int[] RxBuff = new int[256];
int Err = 0;
int HeadLen = 4;    // RX-TX header length
byte[] TxBuff = new byte[144];  // Transmission buffer
int TxHeadLen = 4;             // TX header lenght
int[] TxIntValue = new int[256]; // int value buffer
int TxPeriod = 500;
int TxTimer;
boolean BlinkFlag;
boolean TxFlag = false;

int i = 0;

 float PitchRad=0;  
 float PitchDeg=0;         
 float RollRad=0;
 float RollDeg=0;
 float YawRad=0;
 float YawDeg=0;

 int CircleH = 150;
 int Xh;
 int Yh;
 
 int Gray=150;

 // Compass gauge parameters
 int CmHorX; // X Starting point for compass frame 
 int CmHorY; // Y Starting point for compass frame
 
 // Artificial Horizon gauge parameters
 PGraphics HorizonArea;
 int HA_X, HA_Y;
 int HA_WIDTH = 256, HA_HEIGHT = 256;
 int FrHorX; // X Starting point for artificial horizon frame 
 int FrHorY; // Y Starting point for artificial horizon frame
 int CenterX = HA_WIDTH / 2;
 int CenterY = HA_HEIGHT / 2;
 int SizeX = 600;
 int SizeY = 300;

/*/////////////////////////////////////////////////////////////////////////////*/
void setup()
{
 size(800 , 800, P3D);
 
 // Compass gauge parameters
 CmHorX = width-280;    // X Starting point
 CmHorY = height-280;   // Y Starting point
 
 // Artificial Horizon gauge parameters
 FrHorX = 5;          // X Starting point change this to relocate the gauge
 FrHorY = height-280;   // Y Starting point
 HA_X=FrHorX;  
 HA_Y=FrHorY+20;
 
 // Create a new graphical context
 HorizonArea = createGraphics(HA_WIDTH, HA_HEIGHT, P2D);
 HorizonArea.beginDraw();
 HorizonArea.background(0,0);
 HorizonArea.endDraw();
 
 Xh=-width/2+CircleH/2;
 Yh=height/2-CircleH/2;

 // Load the font that will be used for the N,E,S,W markings
 letters = loadFont("Tunga-48.vlw");
 mono = loadFont("AndaleMono-24.vlw");
 
    i=0;
    println("--------  valid COMM ports are:");

    while(i< Serial.list().length)
    {
      SerialList = (append(SerialList, Serial.list()[i]));
      print(i);
      print("  ");
      println(Serial.list()[i]);
      i++;
    }
   
 //*********************** SET THIS ********************** 
 RS232ComPort = 5;
 RS232Bps = 57600;
 
 try
 { 
      if (RS232FirstInit)
      {
        RS232Port = new Serial(this, Serial.list()[RS232ComPort], RS232Bps);
        RS232FirstInit=false;
      }
      else
      {
       RS232Port.stop(); // if not first initialization, close previous port 
                         // before opening the new one
       RS232Port = new Serial(this, Serial.list()[RS232ComPort], RS232Bps);
      }

      PreInitRS232Flag = false;          // turn ON real RS232 sending
      RS232Port.clear();
 
  }
      catch(Exception e)    // not a valid COM port
      {
            println("NOT A VALID COMM PORT");
            //RS232Port.stop();
            //RS232ComPort = 99;
            PreInitRS232Flag = true;          // turn OFF real RS232 sending
      } 

/*
      int Dummy;
      while (RS232Port.available() > 0) // flush RX buffer
      {
         Dummy = (RS232Port.read());
      }   
*/    
      
}
  
/*/////////////////////////////////////////////////////////////////////////////*/

  
void draw() 
{  
  smooth(8);
  background(Gray);
  
  // Update display
  image(HorizonArea, HA_X, HA_Y);
  
  // communication procedure with UDB4 board using protocol described here:
  // http://www.guiott.com/Rino/CommandDescr/Protocol.htm
  
  if (RxData('K',43))
  {// two bytes -> int16
   // four bytes -> int32
   
   Lat_gps=(RxBuff[HeadLen] << 32) + (RxBuff[HeadLen+1] << 16) + (RxBuff[HeadLen+2] << 8) + (RxBuff[HeadLen+3]); // MSB first
   Lon_gps=(RxBuff[HeadLen+4] << 32) + (RxBuff[HeadLen+5] << 16) + (RxBuff[HeadLen+6] << 8) + (RxBuff[HeadLen+7]);
   Alt_gps=(RxBuff[HeadLen+8] << 32) + (RxBuff[HeadLen+9] << 16) + (RxBuff[HeadLen+10] << 8) + (RxBuff[HeadLen+11]); 
   Sog_gps=Int16toint32(((RxBuff[HeadLen+12] << 8) + (RxBuff[HeadLen+13])));
   Cog_gps=Int16toint32(((RxBuff[HeadLen+14] << 8) + (RxBuff[HeadLen+15])));
   Week_no=Int16toint32(((RxBuff[HeadLen+16] << 8) + (RxBuff[HeadLen+17])));
   Tow=(RxBuff[HeadLen+18] << 32) + (RxBuff[HeadLen+19] << 16) + (RxBuff[HeadLen+20] << 8) + (RxBuff[HeadLen+21]); 
   Hdop=RxBuff[HeadLen+22];
   Svs=RxBuff[HeadLen+23];
   Rmat[0]=Int16toint32(((RxBuff[HeadLen+24] << 8) + (RxBuff[HeadLen+25])));
   Rmat[1]=Int16toint32(((RxBuff[HeadLen+26] << 8) + (RxBuff[HeadLen+27])));
   Rmat[2]=Int16toint32(((RxBuff[HeadLen+28] << 8) + (RxBuff[HeadLen+29])));
   Rmat[3]=Int16toint32(((RxBuff[HeadLen+30] << 8) + (RxBuff[HeadLen+31])));
   Rmat[4]=Int16toint32(((RxBuff[HeadLen+32] << 8) + (RxBuff[HeadLen+33])));
   Rmat[5]=Int16toint32(((RxBuff[HeadLen+34] << 8) + (RxBuff[HeadLen+35])));
   Rmat[6]=Int16toint32(((RxBuff[HeadLen+36] << 8) + (RxBuff[HeadLen+37])));
   Rmat[7]=Int16toint32(((RxBuff[HeadLen+38] << 8) + (RxBuff[HeadLen+39])));
   Rmat[8]=Int16toint32(((RxBuff[HeadLen+40] << 8) + (RxBuff[HeadLen+41])));
   Cpu_load=RxBuff[HeadLen+42];
  
  
      /*
       print("Lat: ");
       print(Lat_gps);
       print("  Long: ");
       print(Lon_gps);
       print("  Alt: ");
       print(Alt_gps);
       print("  Speed: ");
       print(Sog_gps);  
       print("  Dir: ");
       print(Cog_gps);
       print("  Week: ");
       print(Week_no);  
       print("  Time of week: ");
       print(Tow);   
       print("  HDOP: ");
       print(Hdop);  
       print("  Satellites: ");
       print(Svs);  
       print("  Rmat 0: ");
       print(Rmat[0]);  
       print("  Rmat 1: ");
       print(Rmat[1]);   
       print("  Rmat 2: ");
       print(Rmat[2]);            
       print("  Rmat 3: ");
       print(Rmat[3]);                 
       print("  Rmat 4: ");
       print(Rmat[4]);             
       print("  Rmat 5: ");
       print(Rmat[5]);  
       print("  Rmat 6: ");
       print(Rmat[6]);   
       print("  Rmat 7: ");
       print(Rmat[7]);            
       print("  Rmat 8: ");
       println(Rmat[8]);                   
      */ 
 
      PitchRad = asin(Rmat[7] / 16384.0); // RAD
      PitchDeg = PitchRad / (2*PI) * 360;
      RollRad = asin(Rmat[6] / 16385.0);
      RollDeg = RollRad / (2*PI) * 360;
      // Allow for inverted flight
      if (Rmat[8] < 0)
      {
        RollDeg = 180 - RollDeg;
        RollRad = PI - RollRad;
      }
      
      // Calcuate our heading from Rmat readings.
      YawRad = atan2(- Rmat[1] , Rmat[4]);
      YawDeg = (YawRad / (2 * PI)) * 360;
  }
   
  /*
  // just for debug. Simulation of navigation values with the mouse movement 
  float newX = map(mouseX, 0, 300, -150, 150);
  PitchDeg = height% 360-mouseY+180;
  PitchRad=radians(PitchDeg);
  RollDeg = newX % 360;
  RollRad=radians(RollDeg);
  float CompassX = map(mouseX, 0,300, -180,180);
  YawDeg = CompassX % 360;
  YawRad=radians(YawDeg);
  // println("Pitch = " + PitchDeg + "  Roll = " + RollDeg + "  Yaw = " + YawDeg);
  // just for debug. Comment out the above lines for real job
  */
  
  movingHorizon(PitchDeg, RollDeg);
  frameHoriz();
  compass(YawDeg);
  DispText(PitchDeg, RollDeg,YawDeg);
  Box3D();

}
